package cz.cvut.kbss.reporting.service.repository;

import cz.cvut.kbss.reporting.model.Tag;
import cz.cvut.kbss.reporting.persistence.dao.GenericDao;
import cz.cvut.kbss.reporting.persistence.dao.TagDao;
import cz.cvut.kbss.reporting.service.security.SecurityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;

@Service
public class TagService extends AbstractRepositoryService<Tag> {

    private final TagDao dao;
    private final SecurityUtils securityUtils;

    @Override
    protected GenericDao<Tag> getPrimaryDao() {
        return dao;
    }

    @Autowired
    public TagService(TagDao dao, SecurityUtils securityUtils) {
        this.dao = dao;
        this.securityUtils = securityUtils;
    }

    /**
     * Finds tag with the specified name.
     *
     * @param tagName Name of the tag
     * @return Matching tag or {@code null}, if none is found
     */
    public Tag findByName(String tagName) {
        return dao.findByName(tagName);
    }

    @Override
    void prePersist(Tag instance) {
        instance.setAuthor(securityUtils.getCurrentUser());
        instance.setCreated(LocalDateTime.now());
    }
}
