'use strict';

import Reflux from "reflux";
import Actions from "../actions/Actions";
import Ajax from "../utils/Ajax";
import Utils from "../utils/Utils";

const BASE_URL = 'rest/reports';
const BASE_URL_WITH_SLASH = 'rest/reports/';

// When reports are being loaded, do not send the request again
let reportsLoading = false;

export default class ReportStore extends Reflux.Store {

    constructor() {
        super();
        this.state = {
            reports: null,
            currentReport: null
        };
        this.listenables = Actions;
    }

    onLoadAllReports() {
        if (reportsLoading) {
            return;
        }
        reportsLoading = true;
        Ajax.get(BASE_URL).end((data) => {
            reportsLoading = false;
            data = data.map(r => {
                r.created = new Date(r.created);
                r.occurrence.startTime = new Date(r.occurrence.startTime);
                r.occurrence.endTime = new Date(r.occurrence.endTime);
                return r;
            });
            this.setState({reports: data});
        }, () => {
            reportsLoading = false;
            this.setState({reports: []});
        });
    }

    onLoadReport(id, onFinish) {
        Ajax.get(BASE_URL_WITH_SLASH + id).end((data) => {
            this.setState({currentReport: data});
            if (onFinish) {
                onFinish();
            }
        }, () => {
            this.setState({currentReport: null});
            if (onFinish) {
                onFinish();
            }
        });
    }

    onCreateReport(report, onSuccess, onError) {
        Ajax.post(BASE_URL, report).end((data, resp) => {
            if (onSuccess) {
                const id = Utils.extractIdFromLocationHeader(resp);
                onSuccess(id);
            }
            this.onLoadAllReports();
            Actions.reloadTagsIfNecessary(report.tags);
        }, onError);
    }

    onUpdateReport(report, onSuccess, onError) {
        Ajax.put(BASE_URL_WITH_SLASH + report.id).send(report).end(() => {
            if (onSuccess) {
                onSuccess(report.id);
            }
            Actions.reloadTagsIfNecessary(report.tags);
        }, onError);
    }

    onDeleteReport(report, onSuccess, onError) {
        Ajax.del(BASE_URL_WITH_SLASH + report.id).end(() => {
            if (onSuccess) {
                onSuccess();
            }
            this.onLoadAllReports();
        }, onError);
    }

    onClearCurrentReport() {
        this.setState({currentReport: null});
    }
}
